/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

import java.util.Collections;
import java.util.Map;
import java.util.Set;

/**
 * Report element (not necessarily a file). May be marked by a set of aspect tags.
 *
 * @author Sergey Groshev
 */
public abstract class ReportEntity {

  private Map<String, Set<String>> tagMarks;
  private String associatedChannel;

  /**
   * Gets the aspect which contains this report entity. Class {@link
   * com.unitesk.aspectrace.html.ReportGenerator report generator} uses this information for an
   * automatic reference generation; Other components of report generator may use this information
   * as they see fit.
   *
   * @return Aspect name. Empty string is used for an empty aspect. {@code null} if the aspect is
   *     unidentified.
   */
  public String getAspect() {
    return null;
  }

  /** Marks the report element by a set of context tags. */
  public void setTagMarks(Map<String, Set<String>> marks) {
    this.tagMarks = marks;
  }

  /**
   * Gets the set of tags which marks the report element.
   *
   * @return Mapping {@code [aspect +> set of tags]}. Not Null.
   */
  public Map<String, Set<String>> getTagMarks() {
    return tagMarks != null ? tagMarks : Collections.EMPTY_MAP;
  }

  public void setAssociatedChannel(String associatedChannel) {
    this.associatedChannel = associatedChannel;
  }

  public String getAssociatedChannel() {
    return associatedChannel;
  }
}
