/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

/**
 * Trace message, defining tags and synchronized messages. Immutable.
 *
 * @author Sergey Groshev
 */
public final class TagMessage extends AbstractTraceMessage {

  private final String tag;
  private final String parent;
  private final boolean isStart;
  private final boolean isSync;
  private final boolean isUser;

  /** Called only by Tracer and TraceProvider.*/
  public TagMessage(
      String aspect,
      String channel,
      String tag,
      String parent,
      boolean isStart,
      boolean isSync,
      boolean isUser) {
    super(aspect, channel);
    this.tag = tag;
    this.parent = parent;
    this.isStart = isStart;
    this.isSync = isSync;
    this.isUser = isUser;
  }

  @Override
  public Kind getKind() {
    return Kind.TAG_MESSAGE;
  }

  public String getTag() {
    return tag;
  }

  public String getParent() {
    return parent;
  }

  /**
   * For a not synchronized message: determines, whether it is the begin or the end. For a
   * synchronized message the result is undefined.
   *
   * @return {@code true} - tag opens. {@code false} - tag closes.
   */
  public boolean isStart() {
    return isStart;
  }

  /** Determines whether the message is synchronized or not. */
  public boolean isSync() {
    return isSync;
  }

  /**
   * Determines whether the message is user-type or not.
   *
   * @return {@code true} if this tag was set/removed by a trace developer. {@code false} if the tag
   *     source is a trace.
   */
  public boolean isUser() {
    return isUser;
  }

  /* (non-JavaDoc)
   * For debug purposes.*/
  @Override
  public String toString() {
    return "TagMessage "
        + getAspect()
        + ':'
        + tag
        + (isSync ? " sync" : isStart ? " start" : " end")
        + ", channel="
        + getChannel();
  }
}
