/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.common;

import com.unitesk.aspectrace.AspectListener;
import com.unitesk.aspectrace.TagMessage;

/**
 * Interface of the empty aspect message handler.
 *
 * @author Sergey Groshev
 */
public interface CommonListener extends AspectListener {

  /**
   * Notifies about processing start of the trace source.
   *
   * @param traceDesc Definition of the trace source (for common files - filename).
   */
  public void processingStarted(String traceDesc);

  /**
   * Notifies about processing end of the trace source.
   *
   * @param traceDesc Definition of the trace source (for common files - filename).
   * @param normally <code>true</code> if processing ends normally, <code>false</code> otherwise
   *                (IOException, syntax error, etc.)
   */
  public void processingFinished(String traceDesc, boolean normally);

  /**
   * Changes tags and synchronized messages for <b>other</b> aspects. Note: an empty
   * aspect must not have its own tags. If it nonetheless has then they are processed via
   * {@link AspectListener#processTagChange(TagMessage)} method.
   */
  public void processGlobalTagChange(TagMessage msg);

  /** Message which defines a channel. */
  public void processChannelMessage(ChannelMessage msg);
}
