/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.html;

import java.io.PrintWriter;
import java.util.Calendar;

/**
 * Page which has references to all pages (not sections) of it's generator.
 *
 * @author Sergey Groshev
 */
public class SimpleIndexPage extends HtmlReportPage {

  public SimpleIndexPage() {
    super("index.html");
    setTitle("Report index");
  }

  @Override
  public void generatePageBody(PrintWriter pw) {
    pw.println("<h1>All report pages</h1>");
    pw.println("<p>Generated: " + Calendar.getInstance().getTime() + "</p>");
    for (HtmlReportPage p : getGenerator().getHtmlPages()) {
      if (!accept(p)) {
        continue;
      }
      HtmlReportPage hrp = (HtmlReportPage) p;
      pw.print(hrp.generateRef(this, null));
      pw.println("<br/>");
    }
  }

  /* Filter for displayed pages. May be overridden by descendants. Default realization:
   * all pages except this one which are not sections,
   * don't have index page and have not-null path.
   * */

  public boolean accept(HtmlReportPage hrp) {
    return hrp != this
        && hrp.getMasterPage() == null // Calls getParentPage()
        && hrp.getPath() != null;
  }
}
