/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.html;

import com.unitesk.aspectrace.sampleApp.SummaryCollector;
import com.unitesk.aspectrace.templates.DocumentContext;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Page which has references to all pages (not sections) of it's own generator.
 *
 * @author Sergey Groshev
 */
public class TemplateGenericReportsPage extends HtmlReportPage {

  public File reportDir;
  public Map<String, String> summaryInfo = new HashMap<String, String>();
  public ArrayList<String> summaryKeys = new ArrayList<String>();
  public TemplateIndexPage tmpltInd;
  public String aspectName;

  public String getAspectName() {
    return aspectName;
  }

  public void setAspectName(String aspectName) {
    this.aspectName = aspectName;
  }

  public String reportName;
  public String title;
  private String reportType;

  public TemplateGenericReportsPage(
      String name, String aspect, HtmlReportPage ti, String reportType) {
    setPath("" + aspect + ".html");
    title = "" + name + " reports";
    setTitle(title);
    tmpltInd = (TemplateIndexPage) ti;
    aspectName = aspect;
    reportName = name;
    this.reportType = reportType;
  }

  @Override
  public void generatePageHead(PrintWriter pw) {}

  @Override
  public void generatePageBody(PrintWriter pw) {

    pw.println("<h1>" + reportName + " reports</h1>");
    DocumentContext doc = new DocumentContext();
    try {
      Velocity.init();
      VelocityContext vc = new VelocityContext();
      vc.put("doc", doc);
      vc.put("current", this);
      HeaderContext docCont = new HeaderContext(getGenerator(), this);
      vc.put("docContext", docCont);
      SummaryCollector sc = new SummaryCollector();
      vc.put("summary", sc);
      Template tmplt = Velocity.getTemplate(doc.getTemplatePackage() + "genericReports.vm");
      tmplt.merge(vc, pw);

      ReportGenerator rg = new ReportGenerator();
      reportDir = rg.getGenerationDir();

    } catch (Exception ex) {
      // TODO Auto-generated catch block
      ex.printStackTrace();
    }
  }

  /**
   * A filter of displayed pages. May be overridden by descendants. Default realization:
   * all pages except this one which are not sections, don't have master page
   * and have a not-null path.
   */
  public boolean accept(HtmlReportPage hrp) {
    return hrp != this
        && hrp.getMasterPage() == null // Calls getParentPage()
        && hrp.getPath() != null;
  }

  public ArrayList<String> getPages() {
    ArrayList<String> result = new ArrayList<String>();
    for (HtmlReportPage p : getGenerator().getHtmlPages()) {
      if (!accept(p)) {
        continue;
      }

      HtmlReportPage hrp = (HtmlReportPage) p;
      result.add(hrp.getAspect() + hrp.generateRef(this, null));
    }
    return result;
  }

  @Override
  public String getTitle() {
    return "Generated report details";
  }

  public ArrayList<String> getSummaryKeys() {
    return summaryKeys;
  }

  public String getSummaryInfoValue(String key) {
    return summaryInfo.get(key);
  }

  public Map<String, String> getSummaryInfo() {
    return summaryInfo;
  }

  public void setSummaryInfo(Map<String, String> summaryInfo) {
    this.summaryInfo = summaryInfo;
  }

  public String getReportType() {
    return reportType;
  }

  public int getLevel() {
    return 0;
  }
}
