/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.html;

import com.unitesk.aspectrace.sampleApp.SummaryCollector;
import com.unitesk.aspectrace.templates.DocumentContext;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Page which has references to all pages (not sections) of it's own generator.
 *
 * @author Sergey Groshev
 */
public class TemplateIndexPage extends HtmlReportPage {

  public File reportDir;
  public String title = "Generated";
  public Map<String, String> summaryInfo = new HashMap<String, String>();
  public ArrayList<String> summaryKeys = new ArrayList<String>();
  private ArrayList<String> reportTypes = new ArrayList<String>();

  public TemplateIndexPage() {
    setPath("index.html");
    setTitle("Report index");
  }

  @Override
  public void generatePageHead(PrintWriter pw) {}

  @Override
  public void generatePageBody(PrintWriter pw) {

    pw.println("<h1>Test Report</h1>");
    pw.println("<p>Generated: " + Calendar.getInstance().getTime() + "</p>");
    DocumentContext doc = new DocumentContext();
    try {
      Properties props = new Properties();
      props.put(RuntimeConstants.RESOURCE_LOADER, "classpath");
      props.put("classpath.resource.loader.class", ClasspathResourceLoader.class.getName());
      Velocity.init(props);
      String templateName = "overview.vm";
      if (SummaryCollector.isTracingRequirements()) {
        templateName = "requirements.vm";
      }
      reportTypes.add("summary");
      reportTypes.add("details");
      VelocityContext vc = new VelocityContext();
      HeaderContext docCont = new HeaderContext(getGenerator(), this);
      SummaryCollector sc = new SummaryCollector();
      vc.put("doc", doc);
      vc.put("current", this);
      vc.put("docContext", docCont);
      vc.put("summary", sc);
      Template tmplt = Velocity.getTemplate(doc.getTemplatePackage() + templateName);
      tmplt.merge(vc, pw);
      ReportGenerator rg = new ReportGenerator();
      reportDir = rg.getGenerationDir();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public String getReportType() {
    return "summary";
  }

  public int getLevel() {
    return 0;
  }
}
