/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.ref;

import com.unitesk.aspectrace.ReportEntity;

// TODO? Сделать 2 разных параметра класса: для target и для related

/**
 * Поставщик информации о контексте для элементов отчёта. Спроектирован из предположения, что
 * связываемые между собой элементы (контекстные и целевые) имеют один и тот же базовый тип.
 *
 * @author Sergey Groshev
 * @param <T> Общий тип контекстных и целевых элементов.
 */
public interface ContextProvider<T extends ReportEntity> {
  /**
   * Получить контекстную информацию для элемента отчёта, соответствующего текущему (по порядку
   * разбора трассы) событию.
   *
   * @param target Элемент отчёта, для которого запрашивается контекст.
   * @return Элементы отчёта, входящие в текущее окружение. Если подходящего контекста нет -
   *     возвращается null или пустой массив.
   */
  public T[] getRelated(String channel, T target);
}
