/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.requirement;

import com.unitesk.aspectrace.html.HeaderContext;
import com.unitesk.aspectrace.html.HtmlReportPage;
import com.unitesk.aspectrace.html.ReportGenerator;
import com.unitesk.aspectrace.sampleApp.SummaryCollector;
import com.unitesk.aspectrace.templates.DocumentContext;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

public class AllDBReports extends HtmlReportPage {

  public File reportDir;
  public String title = "Generated";
  public Map<String, String> summaryInfo = new HashMap<String, String>();
  public ArrayList<String> summaryKeys = new ArrayList<String>();
  private ArrayList<String> reportTypes = new ArrayList<String>();

  public AllDBReports(String channel) {
    setPath("requirements.html");
    setTitle("Requirements coverage report");
    setAssociatedChannel(channel);
  }

  public AllDBReports() {
    setPath("requirements.html");
    setTitle("Requirements coverage report");
  }

  @Override
  public void generatePageBody(PrintWriter pw) {

    pw.println("<h1>Covered Requirements Report</h1>");
    pw.println("<p>Generated: " + Calendar.getInstance().getTime() + "</p>");
    DocumentContext doc = new DocumentContext();

    try {
      Properties props = new Properties();
      props.put(RuntimeConstants.RESOURCE_LOADER, "classpath");
      props.put("classpath.resource.loader.class", ClasspathResourceLoader.class.getName());
      Velocity.init(props);
      VelocityContext vc = new VelocityContext();

      reportTypes.add("summary");
      reportTypes.add("details");
      HeaderContext docCont = new HeaderContext(getGenerator(), this);
      SummaryCollector sc = new SummaryCollector();

      vc.put("doc", doc);
      vc.put("current", this);
      vc.put("docContext", docCont);
      vc.put("summary", sc);
      Template tmplt = Velocity.getTemplate(doc.getTemplatePackage() + "allDBReports.vm");
      tmplt.merge(vc, pw);

      ReportGenerator rg = new ReportGenerator();
      reportDir = rg.getGenerationDir();

    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public String getReportType() {
    return "requirements";
  }

  public int getLevel() {
    return 0;
  }
}
