/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.requirement;

import com.unitesk.aspectrace.sampleApp.SummaryCollector;

import java.util.HashMap;

// База с глобальной информацией о покрытых требованиях из трассы
// (имя, id, количество passed, failed, uncovered требований)

public class DbInfo {
  public String name;
  public String location;
  public int numPassed;
  public int numFailed;
  public int numUncovered;

  public HashMap<String, ReqInfo> content = new HashMap<String, ReqInfo>();

  public void addToContent(ReqInfo ri) {
    content.put(ri.reqId, ri);
  }

  // изменить вердикт для требования, которое уже встречалось
  public void changeReqContentToFailed(ReqInfo ri) {
    content.remove(ri.reqId);
    content.put(ri.reqId, ri);
  }

  public HashMap<String, ReqInfo> getContent() {
    return content;
  }

  public String getLocation() {
    return location;
  }

  public void setLocation(String location) {
    this.location = location;
  }

  public void setNumUncovered(int numUncovered) {
    this.numUncovered = numUncovered;
  }

  public String getName() {
    return name;
  }

  public int getNumPassed() {
    int result = 0;
    for (ReqInfo ri : content.values()) {
      if (ri.isPassed.equals("passed")) {
        result++;
      }
    }
    return result;
  }

  public int getNumFailed() {
    int result = 0;
    for (ReqInfo ri : content.values()) {
      if (ri.isPassed.equals("failed")) {
        result++;
      }
    }
    return result;
  }

  public int getNumUncovered() {
    int result = SummaryCollector.getAllDbList().get(name).getContent().size();
    for (ReqInfo ri : content.values()) {
      if (ri.isPassed.equals("failed")) {
        result--;
      }
      if (ri.isPassed.equals("passed")) {
        result--;
      }
    }
    return result;
  }

  public DbInfo(String name) {
    this.name = name;
  }

  public DbInfo(String name, int numPassed, int numFailed) {
    this.name = name;
    this.numPassed = numPassed;
    this.numFailed = numFailed;
  }

  public String getLinkToReport(int level) {
    return "<a href=\""
        + SummaryCollector.getRelativePath(level)
        + "requirements/"
        + name
        + ".html\">"
        + name;
  }
}
