/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.sampleApp;

import com.unitesk.aspectrace.TraceProcessorManager;
import com.unitesk.aspectrace.Tracer;
import com.unitesk.aspectrace.coverage.CoverageTracer;
import com.unitesk.aspectrace.error.ErrorTracer;
import com.unitesk.aspectrace.html.ReportGenerator;
import com.unitesk.aspectrace.test.TestTracer;
import com.unitesk.coverage.Coverage;
import com.unitesk.coverage.CoverageBase;
import com.unitesk.coverage.CoverageCombinator;
import com.unitesk.coverage.CoverageElement;
import com.unitesk.coverage.CoverageProcessor;
import com.unitesk.coverage.CoverageStructure;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class CoverageExample {

  private static final String TRACE_NAME = "./my_test.atrace";

  public static void main(String[] args) {
    main1();
    main2();
  }

  public static void main1() {
    try {
      Tracer.getInstance().addXmlTrace(new File(TRACE_NAME));

      // Структуры покрытия
      CoverageStructure myCovStr = CoverageProcessor.toStructure(MyCoverage.class);
      CoverageTracer.traceCoverageStructure(myCovStr);
      CoverageTracer.traceCoverageStructure(QuadB.class);
      CoverageTracer.traceCoverageStructure(Plane.class);
      CoverageTracer.traceCoverageStructure(BranchMpCoverage.class);

      ////// Статический поиск элементов //////

      TestTracer.traceTestStart("static");

      // X1
      CoverageTracer.traceCoverageElement(
          CoverageProcessor.findElement(QuadB.class, QuadB.getPath(1, 0, -5, 7, false)));
      CoverageTracer.traceCoverageElement(MyCoverage.getElement(5, 0, -2, -3, true));
      CoverageTracer.traceCoverageElement(MyCoverage.getElement(5, 2, 1, -3, false));

      // X2
      CoverageTracer.traceCoverageElement(
          CoverageProcessor.findElement(Plane.class, Plane.getPath(-1, 0)));
      CoverageTracer.traceCoverageElement(MyCoverage.getElement(-5, 0));

      // X3
      CoverageTracer.traceCoverageElement(MyCoverage.getElement(-5));
      CoverageTracer.traceCoverageElement(CoverageProcessor.findElement(myCovStr, "X3.POSITIVE"));

      // X5
      CoverageTracer.traceCoverageElement(CoverageProcessor.findElement(MyCoverage.class, "X5"));

      TestTracer.traceTestEnd("static", true);

      ////// Нестатический поиск элементов //////

      TestTracer.traceTestStart("non-static");
      MyCoverage mc;

      // X1
      mc = new MyCoverage();
      Map<String, Object> m = new HashMap<String, Object>();
      m.put("x", Sign.POSITIVE);
      m.put("y", Sign.ZERO);
      mc.X1 =
          CoverageBase.create(
              QuadB.class,
              CoverageBase.create(Plane.class, m),
              CoverageBase.create(Plane.class, Sign.NEGATIVE, Sign.POSITIVE),
              //				CoverageBase.getEmpty(),
              true);
      CoverageTracer.traceCoverageElement(mc);
      CoverageTracer.traceCoverageElement(mc.X1);
      CoverageTracer.traceCoverageElement(MyCoverage.get(-5, 0, -2, -3, true));

      // X2
      mc = new MyCoverage();
      mc.X2 = CoverageBase.create(Plane.class, Sign.NEGATIVE, Sign.POSITIVE);
      CoverageTracer.traceCoverageElement(mc);
      CoverageTracer.traceCoverageElement(mc.X2);
      CoverageTracer.traceCoverageElement(MyCoverage.get(-1, -1));

      // X3
      CoverageTracer.traceCoverageElement(MyCoverage.get(0));

      // X4
      mc = MyCoverage.getX4();
      CoverageTracer.traceCoverageElement(mc);

      // X6
      CoverageTracer.traceCoverageElement(MyCoverage.getX6());

      TestTracer.traceTestEnd("non-static", true);

      // Пример Marked path: статический и нестатический интерфейсы
      TestTracer.traceTestStart("Branch");
      {
        HashMap<Object, Object> map = new HashMap<Object, Object>();
        Object key;
        Object val;

        CoverageTracer.traceCoverageElement(BranchMpCoverage.getVal(null, "", ""));

        CoverageTracer.traceCoverageElement(BranchMpCoverage.getElem(map, key = null, val = ""));
        map.put(key, val);

        CoverageTracer.traceCoverageElement(BranchMpCoverage.getVal(map, key = "", val = null));
        map.put(key, val);

        CoverageTracer.traceCoverageElement(BranchMpCoverage.getElem(map, key = null, val = ""));
        map.put(key, val);

        CoverageTracer.traceCoverageElement(
            BranchMpCoverage.getVal(map, key = "", val = new Integer(10)));
        map.put(key, val);
      }
      TestTracer.traceTestEnd("Branch", true);

      ////// Полный перебор элементов структуры покрытия //////

      TestTracer.traceTestStart("iterator");
      System.out.println("------------------");
      for (MyCoverage c1 : CoverageProcessor.enumerator(MyCoverage.class)) {
        System.out.println(Arrays.toString(c1.getElement().getNamePath()));
        CoverageTracer.traceCoverageElement(c1);
      }
      System.out.println("------------------");
      {
        BranchMpCoverage c2 = CoverageBase.createInit(BranchMpCoverage.class);
        do {
          System.out.println(Arrays.toString(c2.getElement().getNamePath()));
          CoverageTracer.traceCoverageElement(c2);
        } while (c2.increment());
      }
      TestTracer.traceTestEnd("iterator", true);
      System.out.println("------------------");

      System.out.println(
          "MyCoverage: " + CoverageProcessor.countValues(MyCoverage.class) + " values");
      System.out.println(
          "BranchMpCoverage: " + CoverageProcessor.countValues(BranchMpCoverage.class) + " values");

      ////// Работа со строками в CoverageBase (UNION и PRODUCT) //////

      TestTracer.traceTestStart("String");
      {
        String[][] strings = {
          {"a", "b", "+"},
          {"-", "a", "b", "-", "-"},
          {"***", "zzz"}
        };
        for (int i = 0; i < strings.length; i++) {
          Sign sign = Sign.values()[i];
          for (String string : strings[i]) {
            StringProduct s = CoverageBase.create(StringProduct.class, sign, string);
            CoverageTracer.traceCoverageElement(s);
          }
        }

        StringUnion su = new StringUnion();
        su.sign = Sign.ZERO;
        CoverageTracer.traceCoverageElement(su);
        su.sign = Sign.NEGATIVE;
        CoverageTracer.traceCoverageElement(su);
        su.sign = null;
        su.str = "Строка 1";
        CoverageTracer.traceCoverageElement(su);
        su.str = "Строка 2";
        CoverageTracer.traceCoverageElement(su);
      }
      TestTracer.traceTestEnd("String", true);

    } catch (Throwable ex) {
      ex.printStackTrace();
      ErrorTracer.traceFailure("test", "Exception", null, ex);
    } finally {
      // Завершаем трассу
      Tracer.getInstance().endTrace();
    }
  }

  public static void main2() {
    try {
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      // Добавляем входную трассу
      tpm.addTrace(new File(TRACE_NAME));
      // Добавляем компонент, обрабатывающий структуры покрытия
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Добавляем компонент, обрабатывающий тесты
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Запускаем обработку
      tpm.process();
      ////////// Собираем и генерируем HTML-отчёт //////////
      rg.setGenerationDir(new File("./!Report"));
      rg.addResource("/com/unitesk/aspectrace/sampleApp/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable ex) {
      ex.printStackTrace();
    }
  }

  // ------------------------------------------------------

  @Coverage
  public static enum Sign {
    @Coverage(name = "Плюс")
    POSITIVE,
    @Coverage(name = "Ноль")
    ZERO,
    @Coverage(name = "Минус")
    NEGATIVE;

    public static String getPath(int x) {
      return get(x).name();
    }

    public static Sign get(int x) {
      if (x > 0) {
        return POSITIVE;
      } else if (x < 0) {
        return NEGATIVE;
      } else {
        return ZERO;
      }
    }
  }

  @Coverage(comb = CoverageCombinator.PRODUCT)
  public static class Plane extends CoverageBase {
    public Sign x;
    public Sign y;

    public static String getPath(int x, int y) {
      return Sign.getPath(x) + '.' + Sign.getPath(y);
    }

    public static Plane get(int x, int y) {
      return create(Plane.class, Sign.get(x), Sign.get(y));
    }
  }

  @Coverage(comb = CoverageCombinator.PRODUCT)
  public static class QuadB extends CoverageBase {
    @Coverage(prefix = "A: ", flatten = true)
    public Plane a;

    @Coverage(prefix = "B: ", flatten = true)
    public Plane b;

    public boolean dual;

    public static String getPath(int x, int y, int z, int t, boolean b) {
      return Plane.getPath(x, y) + '.' + Plane.getPath(z, t) + '.' + b;
    }

    public static QuadB get(int x, int y, int z, int t, boolean b) {
      return create(QuadB.class, Plane.get(x, y), Plane.get(z, t), b);
    }
  }

  @Coverage(comb = CoverageCombinator.PRODUCT)
  public static class EmptyProduct extends CoverageBase {
    public EmptyUnion a;
    public EmptyUnion b;
    public EmptyUnion c;
  }

  @Coverage(comb = CoverageCombinator.UNION)
  public static class EmptyUnion extends CoverageBase {}

  @Coverage(aspect = "myAspect", name = "Geometry", comb = CoverageCombinator.UNION)
  public static class MyCoverage extends CoverageBase {
    @Coverage(name = "Точка на 4-хмерной плоскости X boolean" /*, flatten=true*/)
    public QuadB X1;

    @Coverage(name = "Точка на плоскости", flatten = true)
    public Plane X2;

    @Coverage(name = "Точка на прямой")
    public Sign X3;

    @Coverage(name = "Особый случай X4")
    public EmptyProduct X4;

    @Coverage(name = "Особый случай X5")
    public EmptyUnion X5;

    @Coverage(name = "Особый случай X6")
    public CoverageBase X6;

    /* Статический интерфейс: статические методы getElement(...),
     * вычисляющие элементы покрытия напрямую или через пути к ним,
     * но не создающие объектов класса.
     */
    public static CoverageElement getElement(int x, int y, int z, int t, boolean b) {
      return CoverageProcessor.findElement(MyCoverage.class, "X1." + QuadB.getPath(x, y, z, t, b));
    }

    public static CoverageElement getElement(int x, int y) {
      return CoverageProcessor.findElement(MyCoverage.class, "X2." + Plane.getPath(x, y));
    }

    public static CoverageElement getElement(int x) {
      return CoverageProcessor.findElement(MyCoverage.class, "X3." + Sign.getPath(x));
    }

    /* Не-статический интерфейс: методы get(...), создающие
     * объекты класса. Далее объекты могут трассироваться с помощью
     * метода CoverageTracer.traceCoverageElement(CoverageBase)
     */
    public static MyCoverage get(int x, int y, int z, int t, boolean b) {
      MyCoverage mc = new MyCoverage();
      mc.X1 = QuadB.get(x, y, z, t, b);
      return mc;
    }

    public static MyCoverage get(int x, int y) {
      MyCoverage mc = new MyCoverage();
      mc.X2 = Plane.get(x, y);
      return mc;
    }

    public static MyCoverage get(int x) {
      MyCoverage mc = new MyCoverage();
      mc.X3 = Sign.get(x);
      return mc;
    }

    public static MyCoverage getX4() {
      MyCoverage mc = new MyCoverage();
      mc.X4 =
          CoverageBase.create(
              EmptyProduct.class, new EmptyUnion(), new EmptyUnion(), new EmptyUnion());
      return mc;
    }

    public static MyCoverage getX5() {
      MyCoverage mc = new MyCoverage();
      mc.X5 = CoverageBase.create(EmptyUnion.class, new EmptyProduct());
      return mc;
    }

    public static MyCoverage getX6() {
      MyCoverage mc = new MyCoverage();
      mc.X6 = CoverageBase.getEmpty();
      return mc;
    }
  }

  @Coverage(
      aspect = "oracle",
      name = "Marked-path coverage for method Map.put()",
      comb = CoverageCombinator.UNION)
  public static class BranchMpCoverage extends CoverageBase {
    @Coverage(name = "Exception thrown")
    public Branch1 branch1;

    @Coverage(name = "Place new value")
    public Branch2 branch2;

    @Coverage(name = "Replacing the old value")
    public Branch3 branch3;

    /** Нестатический интерфейс */
    public static BranchMpCoverage getVal(Map<?, ?> map, Object key, Object val) {
      BranchMpCoverage bmc = new BranchMpCoverage();
      if (map == null) {
        bmc.branch1 = new Branch1();
      } else if (!map.containsKey(key)) {
        bmc.branch2 = Branch2.get(key, val);
      } else {
        bmc.branch3 = Branch3.get(key, val);
      }
      return bmc;
    }

    /** Статический интерфейс */
    public static CoverageElement getElem(Map<?, ?> map, Object key, Object val) {
      StringBuilder markedPath = new StringBuilder();
      String branch;
      if (map == null) {
        branch = "branch1";
      } else {
        markedPath.append('x');
        if (val == null) {
          markedPath.append('v');
        }
        if (key == null) {
          markedPath.append('k');
        }
        if (!map.containsKey(key)) {
          branch = "branch2";
        } else {
          branch = "branch3";
        }
      }
      return CoverageProcessor.findElement(BranchMpCoverage.class, branch + '.' + markedPath);
    }
  }

  @Coverage(comb = CoverageCombinator.UNION)
  public static class Branch extends CoverageBase {}

  public static class Branch1 extends Branch {}

  public static class Branch2 extends Branch {
    @Coverage(name = "Place new value")
    public CoverageBase x;

    @Coverage(name = "Value is null; Place new value")
    public CoverageBase xv;

    @Coverage(name = "Key is null; Place new value")
    public CoverageBase xk;

    @Coverage(name = "Key is null; Value is null; Place new value")
    public CoverageBase xkv;

    // Способ вычисления с кучей дизъюнктов, из которых верен ровно один
    public static Branch2 get(Object key, Object val) {
      return create(Branch2.class, "");
    }
  }

  public static class Branch3 extends Branch {
    @Coverage(name = "Replacing the old value")
    public CoverageBase x;

    @Coverage(name = "Value is null; Replacing the old value")
    public CoverageBase xv;

    @Coverage(name = "Key is null; Replacing the old value")
    public CoverageBase xk;

    @Coverage(name = "Key is null; Value is null; Replacing the old value")
    public CoverageBase xkv;

    // Способ вычисления с ветвящимся вычислением варианта
    public static Branch3 get(Object key, Object val) {
      return create(
          Branch3.class, key != null ? (val != null ? "x" : "xv") : (val != null ? "xk" : "xkv"));
    }
  }

  @Coverage(comb = CoverageCombinator.PRODUCT)
  public static class StringProduct extends CoverageBase {
    public Sign sign;
    public String str;
  }

  @Coverage(comb = CoverageCombinator.UNION)
  public static class StringUnion extends CoverageBase {
    public Sign sign;
    public String str;
  }
}
