/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.sampleApp;

import com.unitesk.aspectrace.TraceNode;
import com.unitesk.aspectrace.requirement.DB;
import com.unitesk.aspectrace.requirement.DbInfo;
import com.unitesk.aspectrace.requirement.ReqInfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class SummaryCollector {

  private static Map<String, ArrayList<TraceNode>> summaryCollector =
      new HashMap<String, ArrayList<TraceNode>>();

  static ArrayList<String> totalCoverages = new ArrayList<String>();

  static String currentTestName = "unknown";
  static String reportPath;

  static boolean tracingRequirements;

  public static boolean isTracingRequirements() {
    return tracingRequirements;
  }

  public static void setTracingRequirements(boolean tracingRequirements) {
    SummaryCollector.tracingRequirements = tracingRequirements;
  }

  public static ArrayList<ReqInfo> reqSummary = new ArrayList<ReqInfo>();

  public static HashMap<String, DbInfo> DbList = new HashMap<String, DbInfo>();

  public static HashMap<String, DB> allDbList = new HashMap<String, DB>();

  public static HashMap<String, DB> getAllDbList() {
    return allDbList;
  }

  public static void addToAllDBList(String idOrName, DB db) {
    allDbList.put(idOrName, db);
  }

  public static String getCurrentTestName() {
    return currentTestName;
  }

  public static void setCurrentTestName(String currentTestName) {
    SummaryCollector.currentTestName = currentTestName;
  }

  public static void setTotalCoverage(String tc) {
    totalCoverages.add(tc);
  }

  public static String getTotalCoverages() {

    // for sum coverage
    String result = "";
    if (totalCoverages.size() > 0) {
      int numerator = 0;
      int denominator = 0;
      for (String str : totalCoverages) {
        numerator += Integer.parseInt(str.substring(0, str.indexOf('/') - 1));
        denominator += Integer.parseInt(str.substring(str.indexOf('/') + 2));
      }

      result = "" + numerator + " / " + denominator;

      if (denominator != 0) {
        result += " (" + (100 * numerator / denominator) + "%)";
      }

    } else {
      result = "0 / 0";
    }

    return result;
  }

  public static Map<String, ArrayList<TraceNode>> getSummaryCollector() {
    return summaryCollector;
  }

  public static void addToSummaryCollector(String key, ArrayList<TraceNode> fsmSummaryStack) {
    SummaryCollector.summaryCollector.put(key, fsmSummaryStack);
  }

  public static Set<String> getKeys() {
    return summaryCollector.keySet();
  }

  public static String getAttrValue(String traceName, String key) {
    String result = "unknown";
    if (SummaryCollector.getSummaryCollector().get(traceName) != null) {
      for (TraceNode propKey : SummaryCollector.getSummaryCollector().get(traceName)) {
        if (propKey != null) {
          if (propKey.getAttr("name").equals(key)) {
            result = propKey.getAttr("value");
          }
        } else {
          result = "unknown";
        }
      }
      if (result == null) {
        result = "unknown";
      }
    }
    return result;
  }

  public static ArrayList<ReqInfo> getReqSummary() {
    return reqSummary;
  }

  public static void setReqSummary(ArrayList<ReqInfo> reqSummary) {
    SummaryCollector.reqSummary = reqSummary;
  }

  public static void addToReqSummary(ReqInfo reqInfo) {
    SummaryCollector.reqSummary.add(reqInfo);
  }

  public static void addToDbList(ReqInfo reqInfo) {
    DbInfo base = DbList.get(reqInfo.db);
    if (base != null) {
      if (reqInfo.isPassed().equals("passed")) {
        base.addToContent(reqInfo);
      } else if (reqInfo.isPassed().equals("failed")) {
        ReqInfo ri = base.getContent().get(reqInfo.reqId);
        if (ri != null) {
          if (ri.isPassed().equals("passed")) {
            base.changeReqContentToFailed(reqInfo);
          }
        } else {
          base.addToContent(reqInfo);
        }
      }
    } else {
      int passed = 0;
      int failed = 0;
      if (reqInfo.isPassed().equals("failed")) {
        failed = 1;
      }
      if (reqInfo.isPassed().equals("passed")) {
        passed = 1;
      }
      DbInfo newBase = new DbInfo(reqInfo.db, passed, failed);
      newBase.addToContent(reqInfo);
      DbList.put(reqInfo.db, newBase);
    }
  }

  public static void setReportPath(String repDir) {
    reportPath = repDir;
  }

  public static String getRelativePath(int level) {
    String result = "";
    for (int i = 0; i < level; i++) {
      result += "../";
    }
    return result;
  }

  public static HashMap<String, DbInfo> getDbList() {
    return DbList;
  }
}
