/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.templates;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Exception with message for logger.
 *
 * @author <a href="mailto:sergg@ispras.ru">Sergey Gomanyuk</a>
 * @version $Revision: 1.3 $
 */
public class LogMessageException extends Exception {
  private static final long SERIAL_VERSION_UID = 1291107347995755134L;

  private final Level level;
  private final Object[] msgObjects;

  public LogMessageException(Level level, String message) {
    super(message);
    this.level = level;
    this.msgObjects = null;
  }

  public LogMessageException(Level level, String message, Throwable cause) {
    super(message, cause);
    this.level = level;
    this.msgObjects = null;
  }

  public LogMessageException(Level level, String message, Object msgObject) {
    super(message);
    this.level = level;
    this.msgObjects = new Object[] {msgObject};
  }

  public LogMessageException(Level level, String message, Object[] msgObjects) {
    super(message);
    this.level = level;
    this.msgObjects = msgObjects;
  }

  public Level getLevel() {
    return level;
  }

  public Object[] getMsgObjects() {
    return msgObjects;
  }

  public void log(Logger lggr) {
    if (getCause() != null && getCause() != this) {
      lggr.log(level, getMessage(), getCause());
    } else if (msgObjects != null) {
      lggr.log(level, getMessage(), msgObjects);
    } else {
      lggr.log(level, getMessage());
    }
  }
}
