/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace.tracer;

import com.unitesk.aspectrace.AbstractTraceMessage;
import com.unitesk.aspectrace.TagMessage;
import com.unitesk.aspectrace.TraceMessage;
import com.unitesk.aspectrace.TraceProcessorManager;
import com.unitesk.aspectrace.TraceProvider;
import com.unitesk.aspectrace.Tracer;

import java.util.Properties;

/**
 * Converts a trace to a trace PUSH-source. Writes the trace which comes from aspect tracers
 * via {@link Tracer} directly to adapters without external representation.
 *
 * @author Sergey Groshev
 */
public class TraceToProcessor extends TraceProvider implements Tracer.Channel {
  public static final String SOURCE_DESC = "<on-the-fly>";

  private TraceProcessorManager tpm;
  private boolean started = false;

  public TraceToProcessor(TraceProcessorManager tpm) {
    this.tpm = tpm;
    tpm.addProvider(this);
  }

  @Override
  public synchronized void startTrace(Properties props) {
    if (started) {
      return;
    }

    attachToManager(tpm);
    fireTraceStart(SOURCE_DESC);
    started = true;
  }

  @Override
  public synchronized void dispatch(AbstractTraceMessage msg)
      throws IllegalStateException, IllegalArgumentException {
    if (!started) {
      throw new IllegalStateException("Tracing has finished or not started yet");
    }

    switch (msg.getKind()) {
      case TRACE_MESSAGE:
        fireMessage((TraceMessage) msg);
        break;
      case TAG_MESSAGE:
        fireTagMessage((TagMessage) msg);
        break;
      case DEBUG_MESSAGE:
        break;
      default:
        throw new IllegalArgumentException("Unsupported message class: " + msg.getClass());
    }
  }

  @Override
  public synchronized void endTrace() {
    if (started) { // May be called more than once
      fireTraceEnd(SOURCE_DESC, true);
    }

    started = false;
  }
}
