/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

import com.unitesk.aspectrace.coverage.CoverageTracer;
import com.unitesk.aspectrace.html.ReportGenerator;
import com.unitesk.aspectrace.sampleApp.HugeCoverageCollector;
import com.unitesk.coverage.Coverage;
import com.unitesk.coverage.CoverageBase;
import com.unitesk.coverage.CoverageCombinator;
import com.unitesk.coverage.CoverageProcessor;
import org.junit.Test;

import java.io.File;

public class HugeCoverageTestCase {

  public static boolean onFly = true;

  @Test
  public void test() {
    try {
      main(new String[] {});
    } catch (Exception ex) {
      System.err.println("Unhandled exception!");
    }
  }

  public static void main(String[] args) throws Exception {
    File trace = onFly ? null : new File("huge_trace.xml");

    TraceProcessorManager tpm = new TraceProcessorManager();
    HugeCoverageCollector hcc = new HugeCoverageCollector();
    tpm.addListener("coverage", hcc);

    if (onFly) {
      Tracer.getInstance().addProcessor(tpm);
    } else {
      Tracer.getInstance().addXmlTrace(trace, "windows-1251");
    }

    CoverageTracer.traceCoverageStructure(BigCoverage.class);
    int step = 0;
    // Template: for each 8 elements 4 are not covered and then 4 are covered twice.
    for (BigCoverage val : CoverageProcessor.enumerator(BigCoverage.class)) {
      step++;
      if (step <= 4) {
        continue;
      }
      CoverageTracer.traceCoverageElement(val);
      CoverageTracer.traceCoverageElement(val);
      if (step >= 8) {
        step = 0;
      }
    }
    Tracer.getInstance().endTrace();

    if (!onFly) {
      tpm.addTrace(trace);
      tpm.process();
    }

    ReportGenerator rg = new ReportGenerator();
    rg.setGenerationDir(new File("./build/test-outputs/HugeCoverageTestCase"));
    rg.setDefaultCharset("windows-1251");
    rg.addPages(hcc.getPages());
    rg.addIndexPage();
    rg.generateReport();
  }

  @Coverage(aspect = "huge", comb = CoverageCombinator.UNION)
  public static class BigCoverage extends CoverageBase {
    public BigCoverage0 singleBranch;
  }

  @Coverage(comb = CoverageCombinator.UNION)
  public static class BigCoverage0 extends CoverageBase {
    public MicroTeskTestCase.InstrSeq3 deepCoverage;
    public MicroTeskTestCase.LoadStoreSits smallEnum1;
    public MicroTeskTestCase.AddrDependency smallEnum2;
    public MicroTeskTestCase.AddDivSits smallEnum3;
  }
}
