/*
 * Copyright 2009-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.aspectrace;

import com.unitesk.aspectrace.html.ReportGenerator;
import com.unitesk.aspectrace.sampleApp.CoverageCollector;
import com.unitesk.aspectrace.sampleApp.TestProcessor;
import org.junit.Test;

import java.io.File;

public class TraceTestCase {

  private static final String TEST_TRACES_PATH = "./src/test/test_traces/";

  @Test
  public void test() {
      main(new String[] {});
  }

  public static void main(String[] args) {
    testGenerateReportAccountReq();
    testGenerateReportTestAccount();
    // testGenerateReportTestCtesk();
    testGenerateReportTraceOk();
    testGenerateReportTraceException();
    testGenerateReportTestbenchFail();
    testGenerateReportTestbenchFail2();
    testGenerateReportTestbenchStartEnd1();
    testGenerateReportTestbenchStartEnd2();
    testGenerateReportTestServer10FailidTrace();
  }

  // ------------------------------------------------------
  public static void testGenerateReportAccountReq() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds new trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "accountReq.xml"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/accountReq").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/accountReq"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestAccount() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "test_account.xml"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/test_account").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/test_account"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  // TODO: Problem with trace.
  public static void testGenerateReportTestCtesk() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "test_ctesk.xml"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/test_ctesk").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/test_ctesk"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTraceException() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "traceException.xml"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/traceException").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/traceException"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTraceOk() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "traceOk.xml"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/traceOk").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/traceOk"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestbenchFail() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "testbench_fail.utt2"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/testbench_fail").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/testbench_fail"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestbenchFail2() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "testbench_fail2.utt2"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/testbench_fail2").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/testbench_fail2"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestbenchStartEnd1() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "testbench_start_end_1.utt2"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/testbench_start_end_1").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/testbench_start_end_1"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestbenchStartEnd2() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "testbench_start_end_2.utt2"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/testbench_start_end_2").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/testbench_start_end_2"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }

  public static void testGenerateReportTestServer10FailidTrace() {
    try {
      System.err.println(">>> Generating report");
      TraceProcessorManager tpm = new TraceProcessorManager();
      ReportGenerator rg = new ReportGenerator();
      rg.setDefaultCharset("windows-1251");
      // Adds input trace.
      tpm.addTrace(new File(TEST_TRACES_PATH + "testServer10_failid_trace.uat"));
      // Adds component which handles coverage structures.
      CoverageCollector cc = new CoverageCollector();
      tpm.addListener("coverage", cc);
      rg.addRefResolver("coverage", cc);
      // Adds component which handles tests.
      TestProcessor tp = new TestProcessor();
      tpm.addListener("test", tp);
      rg.addRefResolver("test", tp);
      rg.addAutoRef("coverage", "test");
      // Starts processing.
      try {
        tpm.process();
      } catch (Exception ex) {
        ex.printStackTrace();
        System.err.println();
        System.err.println("Trying to generate report after failure...");
      }
      ////////// Makes HTML-report. //////////
      new File("./build/test-outputs/TraceTestCase/testServer10_failid_trace").mkdir();
      rg.setGenerationDir(new File("./build/test-outputs/TraceTestCase/testServer10_failid_trace"));
      rg.addResource("./src/main/resources/com/unitesk/aspectrace/report.css", "report.css");
      rg.addIndexPage();
      rg.addPages(cc.getPages());
      rg.addPages(tp.getPages());
      rg.generateReport();
    } catch (Throwable th) {
      th.printStackTrace();
    }
  }
}
